<?php
  
/*
	Forum Cleaner - A MyBB plugin to help Administrators keep things clean.
												  
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * This file is MYBB_ROOT/inc/plugins/forumcleaner.php
 * There should also be a files
 *  MYBB_ROOT/inc/tasks/forumcleaner.php
 *  MYBB_ROOT/inc/languages/english/admin/forumcleaner.lang.php
 *  MYBB_ROOT/inc/languages/english/forumcleaner.lang.php
 */

/*
 * This plugin based on heavily rewritten AutoExpunge plugin (Created by The forum.kde.org team) 
 * and lots of Copy&Paste's from Admin CP tools
 */



##########
# TODO
#


##########
# Changelog
#
# 20100826
#	* initial version
#
# 20100828
#   * added Awaiting Activation and Inactive users deletion
#  
# 20100902
#   * added Deletion of Orphaned avatars
#   * tested for 1.6
#
# 20100907
#   * Message for users about Forum Action added in Forum index and in Forum threads list
#   * fixed bug, which asks for Target Forum in Close Threads action definition
#


// Don't allow direct initialization.
if (! defined('IN_MYBB')) {
	die('Nope.');
}

// The info for this plugin.
function forumcleaner_info() 
{
	global $lang;
	$lang->load('forumcleaner');

	return array(
		'name'			=> $lang->name,
		'avaname'		=> $lang->avaname,
		'description'	=> $lang->description,
		'avadesc'		=> $lang->avadesc,
		'website'		=> 'http://community.mybb.com/thread-77074.html', 
		'author'		=> 'Andriy Smilyanets',
		'authorsite'	=> 'http://community.mybb.com/user-18581.html',
		'version'		=> '2.4.4',
		'compatibility'	=> '16*',
		'guid'			=> 'b61c7ecc40312dc42973d3b06ab611a2',
		'sysname'		=> 'forumcleaner',
		'avasysname'	=> 'orphanavatars',
		'cfglink'		=> 'index.php?module=forum/forumcleaner',
		'avalink'		=> 'index.php?module=user/orphanavatars',
		'files'			=> array(
			'inc/plugins/forumcleaner.php',
			'inc/tasks/forumcleaner.php',
			'inc/languages/english/admin/forumcleaner.lang.php',
			'inc/languages/english/forumcleaner.lang.php',
		),
	);

}

// Hooks.
$plugins->add_hook('admin_forum_menu', 'forumcleaner_admin_forum_menu');
$plugins->add_hook('admin_forum_action_handler', 'forumcleaner_admin_forum_action_handler');
$plugins->add_hook('admin_forum_permissions', 'forumcleaner_admin_forum_permissions');

$plugins->add_hook('admin_user_menu', 'forumcleaner_admin_user_menu');
$plugins->add_hook('admin_user_action_handler', 'forumcleaner_admin_user_action_handler');
$plugins->add_hook('admin_user_permissions', 'forumcleaner_admin_user_permissions');

$plugins->add_hook('admin_load', 'forumcleaner_admin_load');

$plugins->add_hook('build_forumbits_forum','forumcleaner_build_forumbits');
$plugins->add_hook('forumdisplay_start','forumcleaner_build_threadlist');

// Action to take to install the plugin.
function forumcleaner_install() 
{
	global $db,$lang;

	// Retrieve plugin info.
	$me = forumcleaner_info();

	// Create the table for the settings.
	$result = $db->query("
		CREATE TABLE IF NOT EXISTS `" . TABLE_PREFIX . "{$me['sysname']}`
		(
			`xid` INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
			`fid` INT NOT NULL DEFAULT 0,
			`enabled` BOOL NOT NULL DEFAULT 1,
			`threadslist_display` BOOL NOT NULL DEFAULT 0,
			`forumslist_display` BOOL NOT NULL DEFAULT 0,
			`action` VARCHAR(20) NOT NULL,
			`age` INT NOT NULL DEFAULT 0,
			`agetype` VARCHAR(8) NOT NULL,
			`agesecs` INT NOT NULL DEFAULT 0,
			`lastpost` BOOL NOT NULL DEFAULT 1,
			`tofid` INT NOT NULL DEFAULT -1
		)
	");

	$settings_gid = $db->insert_query('settinggroups', array(
		'name' => $me['sysname'],
		'title' => $lang->options_title,
		'description' => $lang->options_desc,
		'disporder' => 40,
	));
	$db->insert_query('settings', array(
		'name' => "{$me['sysname']}_threadlimit",
		'optionscode' => 'text',
		'value' => '30',
		'title' => $lang->threadlimit_title,
		'description' => $lang->sprintf($lang->threadlimit_desc,$me['cfglink']),
		'disporder' => 10,
		'gid' => $settings_gid
	));
	$db->insert_query('settings', array(
		'name' => "{$me['sysname']}_userlimit",
		'optionscode' => 'text',
		'value' => '50',
		'title' => $lang->userlimit_title,
		'description' => $lang->userlimit_desc,
		'disporder' => 20,
		'gid' => $settings_gid
	));
	$db->insert_query('settings', array(
		'name' => "{$me['sysname']}_awaitingdays",
		'optionscode' => 'text',
		'value' => '0',
		'title' => $lang->awaitingdays_title,
		'description' => $lang->awaitingdays_desc,
		'disporder' => 30,
		'gid' => $settings_gid
	));
	$db->insert_query('settings', array(
		'name' => "{$me['sysname']}_inactivedays",
		'optionscode' => 'text',
		'value' => '0',
		'title' => $lang->inactivedays_title,
		'description' => $lang->inactivedays_desc,
		'disporder' => 40,
		'gid' => $settings_gid
	));
	$db->insert_query('settings', array(
		'name' => "{$me['sysname']}_groupids",
		'optionscode' => 'text',
		'value' => '',
		'title' => $lang->groupids_title,
		'description' => $lang->groupids_desc,
		'disporder' => 50,
		'gid' => $settings_gid
	));

	rebuild_settings();

	forumcleaner_addtemplates();

	// Log.
	log_admin_action($me['name']);
}

// Return TRUE if plugin is installed, FALSE otherwise.
function forumcleaner_is_installed() 
{
	global $db;

	$me = forumcleaner_info();

	// Plugin is installed if table exists.
	$result = $db->query("show tables like '" . TABLE_PREFIX . "{$me['sysname']}'");

	if ($db->num_rows($result) > 0) 
	{
		return TRUE;
	}
	else 
	{
		return FALSE;
	}
}

// Action to take to activate the plugin.
function forumcleaner_activate() 
{
	global $message,$lang;

	$me = forumcleaner_info();

	// Add task if task file exists, warn otherwise.
	if (! file_exists(MYBB_ROOT . "inc/tasks/{$me['sysname']}.php")) 
	{
		$message = $lang->sprintf($lang->task_file_not_exist,$me['name'],'<code>inc/tasks/'.$me['sysname'].'.php</code>');
	}
	else 
	{
		forumcleaner_add_task();
	}
	
	// Set admin permissions for tab.
	change_admin_permission('forum', $me['sysname']); 
	change_admin_permission('user', $me['avasysname']); 



	// upgrade from 2.3 to 2.4

	// table upgrade tested only for mysql, 
	// I have no other database engines to test them, so if it not compatible with others, 
	// feel free to modify this or add columns manually
	global $db;

	$query = $db->query("SHOW COLUMNS FROM ".TABLE_PREFIX.$me['sysname']." WHERE field LIKE '%list_display'");

	if ($db->num_rows($query) == 0) 
	{
		// upgrade
		$db->query("ALTER TABLE ".TABLE_PREFIX.$me['sysname'].
			" ADD `threadslist_display` BOOL NOT NULL DEFAULT 0, ADD `forumslist_display` BOOL NOT NULL DEFAULT 0");
	}

	// add templates if not exist
	$query2 = $db->query('SELECT * FROM '.TABLE_PREFIX."templates WHERE title LIKE '".$me['sysname']."_%'");

	if ($db->num_rows($query2) == 0)
	{
		forumcleaner_addtemplates();
	}
	// end of upgrade code

	require_once MYBB_ROOT.'inc/adminfunctions_templates.php';

	//	modify default templates
	find_replace_templatesets('forumbit_depth2_forum','#\\{.modlist}#','{\$forum[\''.$me['sysname'].'_forumbit\']}{\$modlist}');
	find_replace_templatesets('forumdisplay_threadlist','#^#','{\$mybb->input[\''.$me['sysname'].'_threadlist\']}');

}


function forumcleaner_addtemplates()
{
	global $db;

	$me = forumcleaner_info();

	$db->insert_query("templates", array(
		'title' => $me['sysname']."_forumbit",
		'template' => '<br><strong>{$messages}</strong>',
		'sid' => -2,
		'dateline' => TIME_NOW,
	));
	
	$db->insert_query("templates", array(
		'title' => $me['sysname']."_threadlist",
		'template' => '<div class="pm_alert"><strong>{$messages}</strong></div>',
		'sid' => -2,
		'dateline' => TIME_NOW,
	));
}


// Action to take to deactivate the plugin.
function forumcleaner_deactivate() 
{
	global $db, $mybb;

	$me = forumcleaner_info();

	// Remove task.

	// Switch modules and actions.
	$prev_module = $mybb->input['module'];
	$prev_action = $mybb->input['action'];
	$mybb->input['module'] = 'tools/tasks';
	$mybb->input['action'] = 'delete';

	// Fetch ID and title.
	$result = $db->simple_select('tasks', 'tid, title', "file = '{$me['sysname']}'");
	while ($task = $db->fetch_array($result)) 
	{
		// Log.
		log_admin_action($task['tid'], $task['title']);
	}

	// Delete.
	$result = $db->delete_query('tasks', "file = '{$me['sysname']}'");

	// Reset module.
	$mybb->input['module'] = $prev_module;

	// Reset action.
	$mybb->input['action'] = $prev_action;

	// recover templates
	require_once MYBB_ROOT.'inc/adminfunctions_templates.php';
	find_replace_templatesets('forumbit_depth2_forum','#\\{.forum\\[.'.$me['sysname'].'_forumbit.\\]}#','',0);
	find_replace_templatesets('forumdisplay_threadlist','#\\{.mybb->input\\[.'.$me['sysname'].'_threadlist.\\]}#',"",0);

	// Remove admin permission.
	change_admin_permission('forum', $me['sysname'], -1);
	change_admin_permission('user', $me['avasysname'], -1);
}

// Action to take to uninstall the plugin.
function forumcleaner_uninstall() 
{
	global $db, $lang;

	$me = forumcleaner_info();

	// Drop database.
	$result = $db->query('drop table ' . TABLE_PREFIX . $me['sysname']);

	$gid = $db->fetch_field($db->simple_select('settinggroups', 'gid', "name='{$me['sysname']}'"), 'gid');

	if($gid)
	{
		$db->delete_query('settings', 'gid='.$gid);
		$db->delete_query('settinggroups', 'gid='.$gid);
	}
	rebuild_settings();

	// Log.
	log_admin_action($me['name']);

	flash_message($lang->sprintf($lang->plugin_uninstalled, $me['name'], '<ul><li>' . join('</li><li>', $me['files']) . '</li></ul>'), 'success');
}

// Add a menu item in the AdminCP.
function forumcleaner_admin_forum_menu(&$sub_menu) 
{
	$me = forumcleaner_info();
	$sub_menu[] = array(
		'id' => $me['sysname'], 
		'title' => $me['name'], 
		'link' => $me['cfglink'],
	);
}

// The file to use for configuring the plugin.
function forumcleaner_admin_forum_action_handler(&$actions) 
{
	$me = forumcleaner_info();
	$actions[$me['sysname']] = array(
		'active' => $me['sysname'], 
		'file' => 'settings.php'
	);
}

// The text for the entry in the admin permissions page.
function forumcleaner_admin_forum_permissions(&$admin_permissions) 
{
	global $lang;

	$me = forumcleaner_info();
	
	$admin_permissions[$me['sysname']] = $lang->sprintf($lang->can_manage,$me['name']);
}

function get_seconds($age,$type) 
{
	$age_type_secs = array(
		"hours"     => 60*60,
		"days"      => 24*60*60,
		"weeks"     => 7*24*60*60,
		"months"    => 30*24*60*60,
	);

	return $age * $age_type_secs[$type];
}

function forumcleaner_validate_action(&$action) 
{
	global $forum_cache,$lang,$db,$mybb;

	$me = forumcleaner_info(); 

	$errors = array();

	if ($action['age'] == 0) 
	{
		$errors['invalid_age'] = $lang->invalid_age;
	}

	if (!in_array($action['agetype'],array('hours','days','weeks','months')))
	{
		$errors['invalid_agetype'] = $lang->invalid_agetype;
	}

	$action['agesecs'] = get_seconds($action['age'],$action['agetype']);

	if (!in_array($action['action'],array('delete','close','move', 'del_redirects'))) 
	{
		$errors['invalid_action'] = $lang->invalid_action;
	}

	if (! ($action['fid'] == -1 && $action['action'] == 'del_redirects') )
	{ 
		// All forums IS allowed for del_redirects
		// validate other options

		if (!array_key_exists($action['fid'],$forum_cache)) 
		{
			$errors['invalid_forum_id'] = $lang->invalid_forum_id;
		}
		elseif ($forum_cache[$action['fid']]['type'] != 'f')
		{
			$errors['invalid_forum_id'] = $lang->source_category_not_allowed;
		}
	} 

	if ( $action['action'] == 'del_redirects') 
	{
		// doesn't apply to del_redirects
		$action['forumslist_display']=0;
		$action['threadslist_display']=0;
	}

	if ($action['lastpost'] != 1)
	{
		// Lastpost should be 0 or 1; default to 0, don't trigger an error.
		$action['lastpost'] = 0;
	}

	if ( $action['action'] == 'move' ) 
	{
		if (!array_key_exists($action['tofid'],$forum_cache)) 
		{
			$errors['invalid_target_forum_id'] = $lang->invalid_target_forum_id;
		}
		elseif ($forum_cache[$action['tofid']]['type'] != 'f')
		{
			$errors['invalid_target_forum_id'] = $lang->target_category_not_allowed;
		}
	}

	if (count($errors) == 0) {    

		$validate = '';
		if (array_key_exists('xid',$action))
		{
			$result = $db->simple_select($me['sysname'],'*',"xid = '{$action['xid']}'");
			if ($db->num_rows($result)) 
			{
				$validate = "AND xid <> '{$action['xid']}'";
			}
		}        

		// validate for the action age duplications for the forum 
		$result = $db->simple_select($me['sysname'], '*', "
			fid = '{$action['fid']}' 
			AND agesecs = '{$action['agesecs']}' 
			AND lastpost = '{$action['lastpost']}' 
			{$validate}
		");

		if ($db->num_rows($result)) 
		{
			$errors['duplicate_age'] = $lang->duplicate_age;
		}

		// validate for the action duplications for the forum 
		$result = $db->simple_select($me['sysname'], '*', "
			fid = '{$action['fid']}' 
			AND action = '{$action['action']}' 
			{$validate}
		");

		if ($db->num_rows($result)) 
		{
			$errors['duplicate_age'] = $lang->duplicate_action;
		}
	}
	
	return $errors;
}


// simple sort key for forums
function get_sort_key($fid)
{
	global $forum_cache;
		
	if ($fid)
	{
		return get_sort_key($forum_cache[$fid]['pid']).sprintf("%04d",$forum_cache[$fid]['disporder']);
	}
	else
	{
		return '';
	}
}

// to sort action list
function actions_cmp($a,$b) 
{
	$cmp = strcmp($a['sort_key'],$b['sort_key']);
	if ($cmp != 0)
	{
		return $cmp;
	}
	if ($a['agesecs'] < $b['agesecs'])
	{
		return -1;
	}
	if ($a['agesecs'] > $b['agesecs'])
	{
		return 1;
	}
	return 0;
}

// Configuration page.
function forumcleaner_admin_load() 
{
	global $page;

	$me = forumcleaner_info();

	if ($page->active_action == $me['sysname']) 
	{
		forumcleaner_process_forumactions();
	}
	elseif ($page->active_action == $me['avasysname'])
	{
		forumcleaner_process_orphanavatars();
	}
} // function forumcleaner_admin_load() 

function forumcleaner_process_forumactions()
{
	global $db, $page, $mybb, $forum_cache, $lang;

	$me = forumcleaner_info();

	if(!is_array($forum_cache))
	{
		cache_forums();
	}

	$action = (isset($mybb->input['action']) ? $mybb->input['action'] : 'config');

	// silently ignore unknown actions
	if (!in_array($action,array('config','add','addtask','edit','enable','disable','delete')))
	{
		admin_redirect($me['cfglink']);
	}

	$page->add_breadcrumb_item($me['name']);
	$page->output_header($me['name']);

	// Warnings.
	$result = $db->simple_select('tasks', 'tid, enabled, file', "file = '{$me['sysname']}'");
	$task = $db->fetch_array($result);
	if (! file_exists(MYBB_ROOT . "inc/tasks/{$task['file']}.php")) 
	{
		$page->output_alert($lang->sprintf($lang->alert_task_file,
			$me['name'],
			'<code>inc/tasks/'.$me['sysname'].'.php</code>'
		));
	}
	if (! $db->num_rows($result)) 
	{
		$page->output_alert($lang->sprintf($lang->alert_no_task_added,
			$me['name'],
			'<code>inc/tasks/'.$me['sysname'].'.php</code>',
			"{$me['cfglink']}&amp;action=addtask"
		));
	}
	if (! $task['enabled']) 
	{
		$page->output_alert($lang->sprintf($lang->alert_task_disabled,
			$me['name'],
			"index.php?module=tools/tasks&amp;action=enable&amp;tid={$task['tid']}"
		));
	}

	$xid = '-1';

	if (isset($mybb->input['xid']))
	{
		$xid = intval($mybb->input['xid']);
	}

	$db_array = array();

	// silently ignore edit action without xid provided or non-exist xid
	if ($action == 'edit')
	{    
		if ($xid < 0) 
		{
			$action = 'config';
		}
		else 
		{
			$result = $db->simple_select($me['sysname'], '*', "xid = '{$xid}'");
			if ($db->num_rows($result))
			{
				$db_array = $db->fetch_array($result);
			}
			else
			{
				$action = 'config';
			} 
		}
	}

	if ($action == 'addtask') 
	{
		if (forumcleaner_add_task()) 
		{
			flash_message($lang->sprintf($lang->task_added,$me['name']), 'success');
		}
		else 
		{
			flash_message($lang->sprintf($lang->task_exists,$me['name']), 'success');
		}
		admin_redirect($me['cfglink']);
	}

	if ($action == 'delete') 
	{
		if ($xid>=0) 
		{
			$result = $db->simple_select($me['sysname'], '*', "xid = '$xid'");
			if ($expunge = $db->fetch_array($result)) 
			{
				log_admin_action(array('xid' => $xid, 'fid' => $expunge['fid']));
				$db->delete_query($me['sysname'], "xid = '{$xid}'", 1);
			}
			flash_message($lang->action_deleted, 'success');
		}
		admin_redirect($me['cfglink']);
	}

	if ($action == 'disable' or $action == 'enable') 
	{
		if ($xid>=0) 
		{
			$find = 1;
			$update = 0;
			if ($action == 'enable') 
			{
				$find = 0;
				$update = 1;
			}

			$result = $db->simple_select($me['sysname'], '*', "xid = '$xid' and enabled = {$find}");
			if ($expunge = $db->fetch_array($result)) 
			{
				log_admin_action(array('xid' => $xid, 'fid' => $expunge['fid']));
				$db->update_query($me['sysname'], array('enabled' => $update), "xid = '{$xid}'");
			}
		}
		flash_message(($action=='disable'?$lang->action_disabled:$lang->action_enabled), 'success');
		admin_redirect($me['cfglink']);
	}

	$forumactions = array(
		'close'          => $lang->close_threads,
		'delete'         => $lang->delete_threads,
		'move'           => $lang->move_threads,
		'del_redirects'  => $lang->delete_redirects,
	);

	$errors = array();
	$update_array = array();

	// Form received.
	if ($mybb->request_method == "post") {
		if ($xid >= 0) 
		{
			$update_array['xid']=$xid;
			$action = 'edit';
		}
		else 
		{
			$action = 'add';
		}

		$update_array['fid']=$mybb->input['fid'];
		$update_array['age']=$mybb->input['age'];
		$update_array['agetype']=$mybb->input['agetype'];
		$update_array['action']=$mybb->input['forumaction'];
		$update_array['lastpost']=$mybb->input['lastpost'];
		$update_array['forumslist_display']=$mybb->input['forumslist_display'];
		$update_array['threadslist_display']=$mybb->input['threadslist_display'];

		if ($update_array['action'] == 'move' ) 
		{
			$update_array['tofid']=$mybb->input['tofid'];
		}

		$errors = forumcleaner_validate_action($update_array);

		if (count($errors) == 0) 
		{
			// update or insert new action
			if ($xid < 0) 
			{
				// insert
				$db->insert_query($me['sysname'], $update_array);
				$mybb->input['action'] = 'add';
				log_admin_action(array('xid' => $db->insert_id(), 'fid' => $update_array['fid']));
			}
			else
			{
				// update
				unset($update_array['xid']);
				$db->update_query($me['sysname'], $update_array, "xid = '$xid'");
				$mybb->input['action'] = 'update';
				log_admin_action(array('xid' => $xid, 'fid' => $update_array['fid'])); 
			}

			flash_message($lang->sprintf($lang->rules_updated,$me['name']), 'success');
			admin_redirect($me['cfglink']);
		}
	}

	$navtabs = array();
	$navtabs['config'] = array(
		'title'			=> $lang->configuration,
		'link'			=> $me['cfglink'],
		'description'	=> $lang->configuration_desc,
	);

	if ($action == 'edit') 
	{
		$navtabs['edit'] = array(
			'title'			=> $lang->edit_forum_action,
			'link'			=> "{$me['cfglink']}&amp;action=edit&amp;xid={$xid}",
			'description'	=> $lang->edit_forum_action_desc,
		);

		if (!count($update_array))
		{
			$update_array = $db_array;
		};
	}
	else 
	{                
		$navtabs['add'] = array(
			'title'			=> $lang->add_forum_action,
			'link'			=> "{$me['cfglink']}&amp;action=add",
			'description'	=> $lang->add_forum_action_desc,
		);
	};

	$page->output_nav_tabs($navtabs, $action);

	if ($action == 'add' or $action == 'edit') 
	{
		// Create form.
		$form = new Form("{$me['cfglink']}&amp;action={$action}", 'post');

		if ($action == 'edit') 
		{
			echo $form->generate_hidden_field("xid", $xid);
		}

		if (count($errors))
		{
			$page->output_inline_error($errors);
		}

		if (count($update_array)) 
		{
			$update_array['forumaction']=$update_array['action'];
		}
		else
		{
			// clean add
			$update_array = array(
				'fid' => -1,
				'tofid' => -1,
				'forumaction' => 'close',
				'age' => 1,
				'agetype' => 'days',
				'lastpost' => 1,
				'threadslist_display' => 0,
				'forumslist_display' => 0,
			);
		}

		$form_container = new FormContainer($action=='edit'?$lang->edit_forum_action:$lang->add_forum_action);

		$form_container->output_row($lang->source_forum." <em>*</em>", $lang->source_forum_desc, 
		$form->generate_forum_select('fid', $update_array['fid'], array('id' => 'fid', 'main_option' => $lang->none_or_all)), 'fid');

		$agetypes = array(
			"hours"     => $lang->agetype_hours,
			"days"      => $lang->agetype_days,
			"weeks"     => $lang->agetype_weeks,
			"months"    => $lang->agetype_months,
		);
		$form_container->output_row($lang->thread_age, $lang->thread_age_desc, 
		$form->generate_text_box('age', $update_array['age'], array('id' => 'age'))." ".
		$form->generate_select_box('agetype', $agetypes, $update_array['agetype'], array('id' => 'agetype')), 'age');

		$form_container->output_row($lang->thread_post_select, $lang->thread_post_select_desc,
		$form->generate_select_box('lastpost', array(0 => $lang->thread_first_post, 1 => $lang->thread_last_post), 
			$update_array['lastpost'],array('id'=>'lastpost')),'lastpost');

		$form_container->output_row($lang->thread_action, $lang->thread_action_desc,
		$form->generate_select_box('forumaction', $forumactions, $update_array['forumaction'], 
			array('id' => 'forumaction')), 'forumaction');

		$form_container->output_row($lang->target_forum, $lang->target_forum_desc, 
		$form->generate_forum_select('tofid', $update_array['tofid'], array('id' => 'tofid', 'main_option' => $lang->none)), 'tofid');

		$form_container->output_row($lang->forumslist_display, $lang->forumslist_display_desc,
		$form->generate_yes_no_radio('forumslist_display',$update_array['forumslist_display']), 'forumslist_display');		
		
		$form_container->output_row($lang->threadslist_display, $lang->threadslist_display_desc,
		$form->generate_yes_no_radio('threadslist_display',$update_array['threadslist_display']),'threadslist_display');

		$form_container->end();
		// Close form.
		$buttons = array($form->generate_submit_button($lang->save));
		$form->output_submit_wrapper($buttons);
		$form->end();

	}
	else
	{ 
		//config

		// Init table.
		$table = new Table;
		$table->construct_header($lang->forum);
		$table->construct_header($lang->action, array("class" => "align_center"));
		$table->construct_header($lang->age, array("class" => "align_center"));
		$table->construct_header($lang->controls, array("class" => "align_center"));

		// List forums.
		$result = $db->simple_select($me['sysname']);

		if ($db->num_rows($result) > 0) 
		{
			$rows = array();

			while ($row = $db->fetch_array($result)) 
			{
				if ($row['fid']>=0)
				{
					$row['sort_key'] = get_sort_key($row['fid']) . ' ' . $forum_cache[$row['fid']]['name'];
				}
				else
				{
					$row['sort_key'] = '0000';
				}
				array_push($rows,$row);
			}
			// sort actions by forum and treads age
			usort($rows,'actions_cmp');

			foreach ($rows as $row)
			{
				if($row['enabled'])
				{
					$icon = "<a href=\"{$me['cfglink']}&amp;action=disable&amp;xid={$row['xid']}\"><img src=\"styles/{$page->style}/images/icons/tick.gif\" alt=\"{$lang->enabled}\" title=\"{$lang->enabled_title}\" style=\"vertical-align: middle;\" /></a>";
				}
				else
				{
					$icon = "<a href=\"{$me['cfglink']}&amp;action=enable&amp;xid={$row['xid']}\"><img src=\"styles/{$page->style}/images/icons/cross.gif\" alt=\"{$lang->disabled}\" title=\"{$lang->disabled_title}\" style=\"vertical-align: middle;\" /></a>";
				}

				$forum_name = $lang->all_forums;
				if ($row['fid']>=0)
				{
					$forum_name = htmlspecialchars_uni(strip_tags($forum_cache[$row['fid']]['name']));
				}

				$table->construct_cell("<div class=\"float_right\">{$icon}</div><div><strong>".
					"<a href=\"{$me['cfglink']}&amp;action=edit&amp;xid={$row['xid']}\">".
					$forum_name."</a></strong></div>");
				$table->construct_cell($forumactions[$row['action']], array("class" => "align_center"));

				$lang_str = "agetype_{$row['agetype']}";
				$table->construct_cell(
					$lang->sprintf($lang->thread_age_text,
						$row['age'], 
						$lang->$lang_str, 
						$row['lastpost']?$lang->thread_last_post:$lang->thread_first_post
					), 
					array("class" => "align_center")
				);

				$popup = new PopupMenu("action_{$row['xid']}", $lang->options);
				$popup->add_item($lang->edit, "{$me['cfglink']}&amp;action=edit&amp;xid={$row['xid']}");
				if($row['enabled'])
				{
					$popup->add_item($lang->disable,"{$me['cfglink']}&amp;action=disable&amp;xid={$row['xid']}");
				}
				else
				{
					$popup->add_item($lang->enable,"{$me['cfglink']}&amp;action=enable&amp;xid={$row['xid']}");
				}

				$popup->add_item($lang->delete,"{$me['cfglink']}&amp;action=delete&amp;xid={$row['xid']}&amp;my_post_key={$mybb->post_code}",
					"return AdminCP.deleteConfirmation(this, '{$lang->confirm_forum_action_deletion}')");

				$table->construct_cell($popup->fetch(), array("class" => "align_center"));

				$table->construct_row(array('class' => $page->get_alt_bg()));
			}
		}
		else 
		{
			$table->construct_cell($lang->no_forums, array('colspan' => 4));
			$table->construct_row();
		}
		$table->output($lang->forums);
	}
	echo "<br /><small><strong>{$me['name']} plugin &copy; 2010</strong></small>";
	$page->output_footer(TRUE);
	
}// function forumcleaner_process_forumactions()

// Function to add task to task system.
function forumcleaner_add_task() 
{
	global $db, $mybb, $lang;

	require_once MYBB_ROOT . 'inc/functions_task.php';

	$me = forumcleaner_info();

	$result = $db->simple_select('tasks', 'count(tid) as count', "file = '{$me['sysname']}'");
	if (! $db->fetch_field($result, 'count')) 
	{
		// Switch modules and actions.
		$prev_module = $mybb->input['module'];
		$prev_action = $mybb->input['action'];
		$mybb->input['module'] = 'tools/tasks';
		$mybb->input['action'] = 'add';

		// Create task. Have it run every 15 minutes by default.
		$insert_array = array(
			'title'			=> $me['name'],
			'description'	=> $lang->task_desc,
			'file'			=> $me['sysname'],
			'minute'		=> '3,18,33,48',
			'hour'			=> '*',
			'day'			=> '*',
			'month'			=> '*',
			'weekday'		=> '*',
			'lastrun'		=> 0,
			'enabled'		=> 1,
			'logging'		=> 1,
			'locked'		=> 0,
		);
		$insert_array['nextrun'] = fetch_next_run($insert_array);
		$result = $db->insert_query('tasks', $insert_array);
		$tid = $db->insert_id();

		log_admin_action($tid, $me['name']);

		// Reset module and action.
		$mybb->input['module'] = $prev_module;
		$mybb->input['action'] = $prev_action;

		return TRUE;
	}
	return FALSE;
}


// Add a menu item in the AdminCP.
function forumcleaner_admin_user_menu(&$sub_menu) 
{
	$me = forumcleaner_info();
	$sub_menu[] = array(
		'id' => $me['avasysname'], 
		'title' => $me['avaname'], 
		'link' => $me['avalink'],
	);
}

// The file to use for configuring the plugin.
function forumcleaner_admin_user_action_handler(&$actions) 
{
	$me = forumcleaner_info();
	$actions[$me['avasysname']] = array(
		'active' => $me['avasysname'], 
		'file' => 'settings.php'
	);
}

// The text for the entry in the admin permissions page.
function forumcleaner_admin_user_permissions(&$admin_permissions) 
{
	global $lang;

	$me = forumcleaner_info();
	
	$admin_permissions[$me['avasysname']] = $lang->sprintf($lang->can_manage,$me['avaname']);
}


function forumcleaner_process_orphanavatars()
{
	global $db, $page, $mybb, $lang;

	$me = forumcleaner_info();

	$action = (isset($mybb->input['action']) ? $mybb->input['action'] : 'find');

	// silently ignore unknown actions
	if (!in_array($action,array('find','delete')))
	{
		admin_redirect($me['avalink']);
	}

	$page->add_breadcrumb_item($me['avaname']);
	$page->output_header($me['avaname']);

	$files = array();
	$files = find_orphaned_avatars();
	$numfiles = count($files);

	$navtabs = array();
	$navtabs['find'] = array(
		'title'			=> $lang->avafind,
		'link'			=> $me['avalink'],
		'description'	=> $lang->avafind_desc,
	);

	if ($numfiles) 
	{
		$navtabs['delete'] = array(
			'title'			=> $lang->avadelete,
			'link'			=> "{$me['avalink']}&amp;action=delete",
			'description'	=> "",
		);
	}

	$page->output_nav_tabs($navtabs, 'find');

	if ($action == 'find') 
	{ 
		if ($numfiles)
		{
			$page->output_alert($lang->sprintf($lang->avafound, $numfiles));
		}
		else
		{
			$page->output_success($lang->avanotfound);
		}
	}

	if ($action == 'delete') 
	{
		if ($numfiles) 
		{
			$avatarpath = $mybb->settings['avataruploadpath'];

			if(defined('IN_ADMINCP'))
			{
				$avatarpath = '../'.$avatarpath;
			}

			if (! preg_match("#/$#",$avatarpath) ) 
			{
				$avatarpath .= '/';
			}

			foreach ($files as $file) 
			{
				@unlink($avatarpath.$file);
			}

			flash_message($lang->sprintf($lang->avatars_deleted,$numfiles), 'success');
			$files = array();
		}
		else
		{
			flash_message($lang->avatarnotfound, 'error');
		}

		admin_redirect($me['avalink']);
	}

	echo "<br /><small><strong>{$me['name']} plugin &copy; 2010</strong></small>";
	$page->output_footer(TRUE);
	
}// function forumcleaner_process_orphanavatars()



function find_orphaned_avatars()
{
	global $mybb,$db;
	
	if(defined('IN_ADMINCP'))
	{
		$avatarpath = '../'.$mybb->settings['avataruploadpath'];
	}
	else
	{
		$avatarpath = $mybb->settings['avataruploadpath'];
	}

// DEBUG
$debug_avatarfiles = 0;
$debug_avatarrecords = 0;
$debug_matchedfiles = 0;

	$query = $db->query("select uid,avatar from ".TABLE_PREFIX."users where avatartype='upload'");
	$dir = opendir($avatarpath);

	$user_avatars = array();
	$file_avatars = array();

	if($dir)
	{
		$file_notfinished = 1;
		$user_notfinished = 1;

		while ( $file_notfinished || $user_notfinished )
		{
			if ( $file_notfinished and ( $file = @readdir($dir) ) ) 
			{
$debug_avatarfiles ++;

				if ( array_key_exists($file,$user_avatars) )
				{
					unset($user_avatars[$file]);
$debug_matchedfiles ++;
				}
				else
				{
					$file_avatars[$file] = 1;
				}
			}
			else
			{
				$file_notfinished = 0;
			}

			if ( $user_notfinished and ( $ufile = $db->fetch_array($query) ) )
			{
$debug_avatarrecords ++;

				$file = preg_replace("#.+/#","",$ufile['avatar']);

				if ( array_key_exists($file,$file_avatars) )
				{
					unset($file_avatars[$file]);
$debug_matchedfiles ++;
				}
				else
				{
					$user_avatars[$file] = $ufile['avatar'];
				}
			}
			else
			{
				$user_notfinished = 0;
			}
		}

		unset($file_avatars['.']);
		unset($file_avatars['..']);
		unset($file_avatars['index.html']);

$debug_output = fopen($avatarpath . "/forumcleaner.debug.txt", 'w');

fwrite($debug_output, 
"This is DEBUG data for Forum Cleaner orphaned avatars 
avatarpath       = '{$avatarpath}'
avatarfiles      = {$debug_avatarfiles}
avatarrecords    = {$debug_avatarrecords}
matchedfiles     = {$debug_matchedfiles}

files to delete  = " . count($file_avatars) . "
");

foreach ($file_avatars as $file => $ufile) {
    fwrite($debug_output, $file . "\n");
};

fwrite($debug_output, "

orphaned records = " . count($user_avatars) . "
");

foreach ($user_avatars as $file => $ufile) {
    fwrite($debug_output, $file . " => '" . $ufile . "'\n");
};

fclose($debug_output);


		return array_keys($file_avatars);
	}
	else
	{
		return array();
	}
}



// forms message to user with Forum Action description
function get_forumaction_desc($fid, $where)
{
	static $forumaction_cache;
	global $db, $lang, $forum_cache;

	$me = forumcleaner_info();


	// keep information cached
	if (!is_array($forumaction_cache))
	{
		$query = $db->simple_select($me['sysname'],'*','enabled = 1 AND (forumslist_display = 1 OR threadslist_display = 1)');
		
		$forumaction_cache = array();
		while ($fa = $db->fetch_array($query))
		{
			if ( $fa['forumslist_display'] == 1 )
			{
				$forumaction_cache['forums'][$fa['fid']][$fa['agesecs']] = $fa;
			}
			if ( $fa['threadslist_display'] == 1 )
			{
				$forumaction_cache['threads'][$fa['fid']][$fa['agesecs']] = $fa;
			}
		}
	}

	$agetypes = array(
		"hours"     => $lang->topics_hours,
		"days"      => $lang->topics_days,
		"weeks"     => $lang->topics_weeks,
		"months"    => $lang->topics_months,
	);

	$forumactions = array(
		'close'  => $lang->topics_closed,
		'delete' => $lang->topics_deleted,
		'move'   => $lang->topics_moved,
	);

	$lastpost = array(
		0	=> $lang->topics_firstpost,
		1	=> $lang->topics_lastpost,
	);


	if (count($forumaction_cache) and 
		count($forumaction_cache[$where]) and 
		array_key_exists($fid,$forumaction_cache[$where]))
	{
		$actions = array();
		$actions = $forumaction_cache[$where][$fid];
		ksort($actions,SORT_NUMERIC);

		$ret = '';
		$comma = '';
		foreach ($actions as $agesecs => $fa) 
		{
			$ret .= $comma;
			if ($fa['action'] == 'close' or $fa['action'] == 'delete')
			{
				$ret .= $lang->sprintf($forumactions[$fa['action']],
					$lastpost[$fa['lastpost']], 
					$fa['age'], 
					$agetypes[$fa['agetype']]
				);
			}
			else
			{
				if(!is_array($forum_cache))
				{
					cache_forums();
				}
				
				$ret .= $lang->sprintf($forumactions['move'],
					htmlspecialchars_uni(strip_tags($forum_cache[$fa['tofid']]['name'])),
					$lastpost[$fa['lastpost']], 
					$fa['age'], 
					$agetypes[$fa['agetype']]
				);
			}
			$comma = '<br>';
		}
		return $ret;
	}
	else
	{
		return '';
	}
}


function forumcleaner_build_forumbits($forum) 
{
	global $templates;
	$me = forumcleaner_info();

	$messages = get_forumaction_desc($forum['fid'],'forums');
	if (strlen($messages)) 
	{
		$subst = '';

		eval("\$subst = \"".$templates->get($me['sysname']."_forumbit")."\";");

		$forum[$me['sysname'].'_forumbit'] = $subst;
	}
}

function forumcleaner_build_threadlist()
{
	global $templates,$mybb;
	$me = forumcleaner_info();

	$messages = get_forumaction_desc($mybb->input['fid'],'threads');
	if (strlen($messages)) 
	{
		$subst = '';

		eval("\$subst = \"".$templates->get($me['sysname']."_threadlist")."\";");

		$mybb->input[$me['sysname'].'_threadlist'] = $subst;
	}
}

?>
